/*==============================================================================*/
/*  Copyright (C) 2021, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT+ Ver1.00～                                                    */
/*  FileID :   62a                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include "vxWorks.h"
#include "types.h"
#include "taskLib.h"
#include <stdio.h>

#define DT_USE_CLOCK_GETTIME	1
#if DT_USE_CLOCK_GETTIME
#include "timers.h"
#else
#include "tickLib.h"
#endif

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define DT_VARIABLE_BIT		0x2

/*==============================================================================*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:   Please set 1 when you add Event Value to Test Point.                */
/*==============================================================================*/
#define DT_ADD_EVENT_INFO	1

static FILE *dat_file = NULL;
static int BusoutMode = 0;
unsigned int pid = 0;

#if DT_USE_CLOCK_GETTIME
#else
unsigned long long start_tick
#endif


/*==============================================================================*/
/*  Macro:  BusOutModeSet                                                       */
/*  Desc:   0 => open file 1 => writing 2 => close file                         */
/*==============================================================================*/
#define BusOutModeSet(mode) BusoutMode = mode

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut(DT_UINT addr, DT_UINT dat)
{
	unsigned char buff[16] = {0};
	struct timespec now;
	unsigned long long time;

#if DT_ADD_EVENT_INFO
	pid = taskIdSelf();
#endif
	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
#if DT_USE_CLOCK_GETTIME
#else
		start_tick = tickGet();
#endif
		BusOutModeSet(1);
	}

	buff[0] = (unsigned char)((dat) & 0xFF);
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);
	buff[2] = (unsigned char)((addr) & 0xFF);
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);

	/* time data */
#if DT_USE_CLOCK_GETTIME
	/* Nanoseconds */
	if (clock_gettime(CLOCK_REALTIME, &now) == -1) {
		printf("DT : timer error!\n");
	}
	buff[4] = (unsigned char)((now.tv_nsec) & 0xFF);
	buff[5] = (unsigned char)((now.tv_nsec >> 8) & 0xFF);
	buff[6] = (unsigned char)((now.tv_nsec >> 16) & 0xFF);
	buff[7] = (unsigned char)((now.tv_nsec >> 24) & 0xFF);
	buff[8] = (unsigned char)((now.tv_sec) & 0xFF);
	buff[9] = (unsigned char)((now.tv_sec >> 8) & 0xFF);
	buff[10] = (unsigned char)((now.tv_sec >> 16) & 0xFF);
	buff[11] = (unsigned char)((now.tv_sec >> 24) & 0xFF);
#else
 	/* Microseconds */
	time = ((tickGet() - start_tick) * 1000 * 1000) / sysClkRateGet();
	buff[4] = (unsigned char)((time) & 0xFF);
	buff[5] = (unsigned char)((time >> 8) & 0xFF);
	buff[6] = (unsigned char)((time >> 16) & 0xFF);
	buff[7] = (unsigned char)((time >> 24) & 0xFF);
	buff[8] = (unsigned char)((time >> 32) & 0xFF);
	buff[9] = (unsigned char)((time >> 40) & 0xFF);
	buff[10] = (unsigned char)((time >> 48) & 0xFF);
	buff[11] = (unsigned char)((time >> 56) & 0xFF);
#endif

#if DT_ADD_EVENT_INFO
	/* pid */
	buff[12] = (unsigned char)((pid) & 0xFF);
	buff[13] = (unsigned char)((pid >> 8) & 0xFF);
	buff[14] = (unsigned char)((pid >> 16) & 0xFF);
	buff[15] = (unsigned char)((pid >> 24) & 0xFF);
#else
	/* Reserve */
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;
#endif

	fwrite(buff, sizeof(char), 16, dat_file);
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput(DT_UINT addr, DT_UINT dat, void *value, DT_UINT size)
{
	char buff[16];
	char *p = value;
	int i, j, packetNum, count;

	if (size > 256) size = 256;
	packetNum = (size + 7) / 8;

	_TP_BusOut((addr | DT_VARIABLE_BIT), dat);	/* Memory Dump Start */

	buff[0] = size;
	buff[1] = packetNum;
	buff[2] = 0x03;
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;

	count = 0;
	for (i = 0; i < packetNum; i++) {
		buff[3] = i;
		for (j = 0; j < 8; j++, count++, p++) {
			buff[4+j] = (count < size)?*p:0x00;
		}
		fwrite(buff, sizeof(char), 16, dat_file);
	}
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}
